region_mappings = {
    # Asia Pacific
    "CN": "cn-hongkong",  # China -> Hong Kong
    "HK": "cn-hongkong",  # Hong Kong
    "TW": "cn-hongkong",  # Taiwan -> Hong Kong
    "MO": "cn-hongkong",  # Macau -> Hong Kong
    # Southeast Asia
    "SG": "ap-southeast-1",  # Singapore
    "MY": "ap-southeast-3",  # Malaysia
    "ID": "ap-southeast-5",  # Indonesia
    "TH": "ap-southeast-1",  # Thailand -> Singapore
    "VN": "ap-southeast-1",  # Vietnam -> Singapore
    "PH": "ap-southeast-1",  # Philippines -> Singapore
    "BN": "ap-southeast-1",  # Brunei -> Singapore
    "KH": "ap-southeast-1",  # Cambodia -> Singapore
    "LA": "ap-southeast-1",  # Laos -> Singapore
    "MM": "ap-southeast-1",  # Myanmar -> Singapore
    # Australia/Oceania
    "AU": "ap-southeast-2",  # Australia
    "NZ": "ap-southeast-2",  # New Zealand -> Australia
    "FJ": "ap-southeast-2",  # Fiji -> Australia
    "PG": "ap-southeast-2",  # Papua New Guinea -> Australia
    # South Asia
    "IN": "ap-south-1",  # India
    "LK": "ap-south-1",  # Sri Lanka -> India
    "PK": "ap-south-1",  # Pakistan -> India
    "BD": "ap-south-1",  # Bangladesh -> India
    "NP": "ap-south-1",  # Nepal -> India
    "BT": "ap-south-1",  # Bhutan -> India
    "MV": "ap-south-1",  # Maldives -> India
    # Middle East
    "AE": "me-east-1",  # UAE
    "SA": "me-east-1",  # Saudi Arabia -> UAE
    "QA": "me-east-1",  # Qatar -> UAE
    "KW": "me-east-1",  # Kuwait -> UAE
    "BH": "me-east-1",  # Bahrain -> UAE
    "OM": "me-east-1",  # Oman -> UAE
    "IL": "me-east-1",  # Israel -> UAE
    "TR": "me-east-1",  # Turkey -> UAE
    # Europe
    "DE": "eu-central-1",  # Germany
    "FR": "eu-central-1",  # France -> Germany
    "IT": "eu-central-1",  # Italy -> Germany
    "ES": "eu-central-1",  # Spain -> Germany
    "GB": "eu-central-1",  # UK -> Germany
    "IE": "eu-central-1",  # Ireland -> Germany
    "NL": "eu-central-1",  # Netherlands -> Germany
    "BE": "eu-central-1",  # Belgium -> Germany
    "CH": "eu-central-1",  # Switzerland -> Germany
    "AT": "eu-central-1",  # Austria -> Germany
    "PL": "eu-central-1",  # poland -> Germany
    # North America
    "US": "us-east-1",  # USA
    "CA": "us-east-1",  # Canada -> US East
    "MX": "us-west-1",  # Mexico -> US West
    # South America
    "BR": "us-east-1",  # Brazil -> US East
    "AR": "us-east-1",  # Argentina -> US East
    "CL": "us-east-1",  # Chile -> US East
    "CO": "us-east-1",  # Colombia -> US East
    "PE": "us-east-1",  # Peru -> US East
}



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pymammotion/proto/basestation.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#pymammotion/proto/basestation.proto\"2\n\x1arequest_basestation_info_t\x12\x14\n\x0crequest_type\x18\x01 \x01(\r\"\\\n\nbase_score\x12\x12\n\nbase_score\x18\x01 \x01(\r\x12\x11\n\tbase_leve\x18\x02 \x01(\r\x12\x12\n\nbase_moved\x18\x03 \x01(\r\x12\x13\n\x0b\x62\x61se_moving\x18\x04 \x01(\r\"\xe6\x02\n\x1bresponse_basestation_info_t\x12\x15\n\rsystem_status\x18\x01 \x01(\x04\x12\x18\n\x10\x61pp_connect_type\x18\x02 \x01(\r\x12\x10\n\x08\x62le_rssi\x18\x03 \x01(\x05\x12\x11\n\twifi_rssi\x18\x04 \x01(\x05\x12\x10\n\x08sats_num\x18\x05 \x01(\x04\x12\x11\n\tlora_scan\x18\x06 \x01(\x03\x12\x14\n\x0clora_channel\x18\x07 \x01(\x03\x12\x12\n\nlora_locid\x18\x08 \x01(\x03\x12\x12\n\nlora_netid\x18\t \x01(\x03\x12\x12\n\nrtk_status\x18\n \x01(\x04\x12\x17\n\x0flowpower_status\x18\x0b \x01(\x05\x12\x17\n\x0fmqtt_rtk_status\x18\x0c \x01(\x05\x12\x13\n\x0brtk_channel\x18\r \x01(\x05\x12\x12\n\nrtk_switch\x18\x0e \x01(\x05\x12\x1f\n\nscore_info\x18\x0f \x01(\x0b\x32\x0b.base_score\"{\n\x16\x61pp_to_base_mqtt_rtk_t\x12\x12\n\nrtk_switch\x18\x01 \x01(\x05\x12\x0f\n\x07rtk_url\x18\x02 \x01(\t\x12\x10\n\x08rtk_port\x18\x03 \x01(\x05\x12\x14\n\x0crtk_username\x18\x04 \x01(\t\x12\x14\n\x0crtk_password\x18\x05 \x01(\t\"3\n\x16\x62\x61se_to_app_mqtt_rtk_t\x12\x19\n\x11rtk_switch_status\x18\x01 \x01(\x05\"\xfc\x01\n\x0b\x42\x61seStation\x12-\n\x06to_dev\x18\x01 \x01(\x0b\x32\x1b.request_basestation_info_tH\x00\x12.\n\x06to_app\x18\x02 \x01(\x0b\x32\x1c.response_basestation_info_tH\x00\x12;\n\x18\x61pp_to_base_mqtt_rtk_msg\x18\x03 \x01(\x0b\x32\x17.app_to_base_mqtt_rtk_tH\x00\x12;\n\x18\x62\x61se_to_app_mqtt_rtk_msg\x18\x04 \x01(\x0b\x32\x17.base_to_app_mqtt_rtk_tH\x00\x42\x14\n\x12\x42\x61seStationSubTypeb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'pymammotion.proto.basestation_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _REQUEST_BASESTATION_INFO_T._serialized_start=39
  _REQUEST_BASESTATION_INFO_T._serialized_end=89
  _BASE_SCORE._serialized_start=91
  _BASE_SCORE._serialized_end=183
  _RESPONSE_BASESTATION_INFO_T._serialized_start=186
  _RESPONSE_BASESTATION_INFO_T._serialized_end=544
  _APP_TO_BASE_MQTT_RTK_T._serialized_start=546
  _APP_TO_BASE_MQTT_RTK_T._serialized_end=669
  _BASE_TO_APP_MQTT_RTK_T._serialized_start=671
  _BASE_TO_APP_MQTT_RTK_T._serialized_end=722
  _BASESTATION._serialized_start=725
  _BASESTATION._serialized_end=977
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass
from typing import Any, Generic, Literal, TypeVar, Union

from mashumaro import DataClassDictMixin
from mashumaro.mixins.orjson import DataClassORJSONMixin

DataT = TypeVar("DataT")


@dataclass
class Item(DataClassDictMixin, Generic[DataT]):
    time: int
    value: DataT


@dataclass
class BatteryPercentageItems(DataClassORJSONMixin):
    batteryPercentage: int


@dataclass
class BMSHardwareVersionItems(DataClassORJSONMixin):
    bmsHardwareVersion: str


@dataclass
class CoordinateItems(DataClassORJSONMixin):
    coordinate: str  # '{"lon":0.303903,"lat":1.051868}'


@dataclass
class DeviceStateItems(DataClassORJSONMixin):
    deviceState: int


@dataclass
class DeviceVersionItems(DataClassORJSONMixin):
    deviceVersion: str


@dataclass
class DeviceVersionInfoItems(DataClassORJSONMixin):
    deviceVersionInfo: str


@dataclass
class ESP32VersionItems(DataClassORJSONMixin):
    esp32Version: str


@dataclass
class LeftMotorBootVersionItems(DataClassORJSONMixin):
    leftMotorBootVersion: str


@dataclass
class LeftMotorVersionItems(DataClassORJSONMixin):
    leftMotorVersion: str


@dataclass
class MCBootVersionItems(DataClassORJSONMixin):
    mcBootVersion: str


@dataclass
class NetworkInfoItems(DataClassORJSONMixin):
    networkInfo: str


@dataclass
class RightMotorBootVersionItems(DataClassORJSONMixin):
    rightMotorBootVersion: str


@dataclass
class RightMotorVersionItems(DataClassORJSONMixin):
    rightMotorVersion: str


@dataclass
class RTKVersionItems(DataClassORJSONMixin):
    rtkVersion: str


@dataclass
class StationRTKVersionItems(DataClassORJSONMixin):
    stationRtkVersion: str


@dataclass
class STM32H7VersionItems(DataClassORJSONMixin):
    stm32H7Version: str


@dataclass
class OTAProgressItems(DataClassORJSONMixin):
    result: int
    otaId: str
    progress: int
    message: str
    version: str
    properties: str


ItemTypes = Union[
    BatteryPercentageItems,
    BMSHardwareVersionItems,
    CoordinateItems,
    DeviceStateItems,
    DeviceVersionItems,
    DeviceVersionInfoItems,
    ESP32VersionItems,
    LeftMotorBootVersionItems,
    LeftMotorVersionItems,
    MCBootVersionItems,
    NetworkInfoItems,
    RightMotorBootVersionItems,
    RightMotorVersionItems,
    RTKVersionItems,
    StationRTKVersionItems,
    STM32H7VersionItems,
    OTAProgressItems,
]


@dataclass
class Item:
    time: int
    value: int | float | str | dict[str, Any] | ItemTypes  # Depending on the type of value


@dataclass
class Items:
    iotState: Item | None = None
    extMod: Item | None = None
    deviceVersionInfo: Item | None = None
    leftMotorBootVersion: Item | None = None
    knifeHeight: Item | None = None
    rtMrMod: Item | None = None
    iotMsgHz: Item | None = None
    iotMsgTotal: Item | None = None
    loraRawConfig: Item | None = None
    loraGeneralConfig: Item | None = None
    leftMotorVersion: Item | None = None
    intMod: Item | None = None
    coordinate: Item | None = None
    bmsVersion: Item | None = None
    rightMotorVersion: Item | None = None
    stm32H7Version: Item | None = None
    rightMotorBootVersion: Item | None = None
    deviceVersion: Item | None = None
    rtkVersion: Item | None = None
    ltMrMod: Item | None = None
    networkInfo: Item | None = None
    bmsHardwareVersion: Item | None = None
    batteryPercentage: Item | None = None
    deviceState: Item | None = None
    deviceOtherInfo: Item | None = None
    mcBootVersion: Item | None = None
    otaProgress: Item | None = None


@dataclass
class Params(DataClassORJSONMixin):
    deviceType: Literal["LawnMower"]
    checkFailedData: dict
    groupIdList: list[str]
    _tenantId: str
    groupId: str
    categoryKey: Literal["LawnMower"]
    batchId: str
    gmtCreate: int
    productKey: str
    generateTime: int
    deviceName: str
    _traceId: str
    iotId: str
    JMSXDeliveryCount: int
    checkLevel: int
    qos: int
    requestId: str
    _categoryKey: str
    namespace: str
    tenantId: str
    thingType: Literal["DEVICE"]
    items: Items
    tenantInstanceId: str


@dataclass
class ThingPropertiesMessage(DataClassORJSONMixin):
    method: Literal["thing.properties"]
    id: str
    params: Params
    version: Literal["1.0"]



# === sendOrderMsg_Media ===
from abc import ABC

from pymammotion.mammotion.commands.abstract_message import AbstractMessage
from pymammotion.proto import (
    MUL_LANGUAGE,
    MUL_SEX,
    LubaMsg,
    MsgAttr,
    MsgCmdType,
    MsgDevice,
    MulSetAudio,
    MulSetWiper,
    SocMul,
)


class MessageMedia(AbstractMessage, ABC):
    def send_order_msg_media(self, mul):
        luba_msg = LubaMsg(
            msgtype=MsgCmdType.MUL,
            sender=MsgDevice.DEV_MOBILEAPP,
            rcver=self.get_msg_device(MsgCmdType.MUL, MsgDevice.SOC_MODULE_MULTIMEDIA),
            msgattr=MsgAttr.REQ,
            seqs=self.seqs.increment_and_get() & 255,
            version=1,
            subtype=self.user_account,
            mul=mul,
        )

        return luba_msg.SerializeToString()

    def set_car_volume(self, volume: int):
        """Set the car volume. 0 - 100"""
        return self.send_order_msg_media(SocMul(set_audio=MulSetAudio(at_switch=volume)))

    def set_car_voice_language(self, language_type: MUL_LANGUAGE | str | None):
        return self.send_order_msg_media(SocMul(set_audio=MulSetAudio(au_language=language_type)))

    def set_car_volume_sex(self, sex: MUL_SEX):
        return self.send_order_msg_media(SocMul(set_audio=MulSetAudio(sex=sex)))

    def set_car_wiper(self, round_num: int):
        """Set mower wiper."""
        # 2
        return self.send_order_msg_media(SocMul(set_wiper=MulSetWiper(round=round_num)))

    # TODO add lamp methods



from threading import Lock


class AtomicInteger:
    """Thread-safe atomic integer implementation."""

    def __init__(self, initial_value: int = 0) -> None:
        """Initialize atomic integer with given value."""
        self._value = initial_value
        self._lock = Lock()

    def get(self) -> int:
        """Get the current value."""
        with self._lock:
            return self._value

    def set(self, value: int) -> None:
        """Set a new value."""
        with self._lock:
            self._value = value

    def increment_and_get(self) -> int:
        """Increment the value and return the new value."""
        with self._lock:
            self._value += 1
            return self._value

    def decrement_and_get(self) -> int:
        """Decrement the value and return the new value."""
        with self._lock:
            self._value -= 1
            return self._value

    def add_and_get(self, delta: int) -> int:
        """Add delta to value and return the new value."""
        with self._lock:
            self._value += delta
            return self._value

    def compare_and_set(self, expect: int, update: int) -> bool:
        """Compare value with expected and set to update if they match."""
        with self._lock:
            if self._value == expect:
                self._value = update
                return True
            return False

    def __str__(self) -> str:
        """Returns string representation of the atomic integer."""
        return str(self.get())

    def __repr__(self) -> str:
        """Detailed string representation of the atomic integer."""
        return f"AtomicInteger({self.get()})"



from bleak import BleakClient, BleakScanner, BLEDevice
from bleak.backends.characteristic import BleakGATTCharacteristic

from pymammotion.bluetooth.const import SERVICE_CHANGED_CHARACTERISTIC, UUID_NOTIFICATION_CHARACTERISTIC
from pymammotion.event.event import BleNotificationEvent


class MammotionBLE:
    """Class for basic ble connections to mowers."""

    client: BleakClient

    def __init__(self, bleEvt: BleNotificationEvent) -> None:
        self._bleEvt = bleEvt

    async def scanForLubaAndConnect(self) -> bool:
        scanner = BleakScanner()

        def scanCallback(device, advertising_data) -> bool:
            # TODO: do something with incoming data
            print(device)
            print(advertising_data)
            if advertising_data.local_name and (
                "Luba-" in advertising_data.local_name or "Yuka-" in advertising_data.local_name
            ):
                return True
            return False

        device = await scanner.find_device_by_filter(scanCallback)
        if device is not None:
            return await self.create_client(device)
        return False

    async def create_client(self, device: BLEDevice):
        self.client = BleakClient(device.address)
        return await self.connect()

    async def connect(self) -> bool:
        return await self.client.connect() if self.client is not None else False

    async def disconnect(self) -> bool:
        return await self.client.disconnect() if self.client is not None else False

    async def notification_handler(self, _characteristic: BleakGATTCharacteristic, data: bytearray) -> None:
        """Simple notification handler which prints the data received."""
        await self._bleEvt.BleNotification(data)

    def service_changed_handler(self, characteristic: BleakGATTCharacteristic, data: bytearray) -> None:
        """Simple notification handler which prints the data received."""
        print(f"Response 2 {characteristic.description}: {data}")
        print(data.decode("utf-8"))
        # BlufiNotifyData
        # run an event handler back to somewhere

    async def notifications(self) -> None:
        if self.client.is_connected:
            await self.client.start_notify(UUID_NOTIFICATION_CHARACTERISTIC, self.notification_handler)
            await self.client.start_notify(SERVICE_CHANGED_CHARACTERISTIC, self.service_changed_handler)

    def get_client(self):
        """Returns the ble client."""
        return self.client



from enum import IntEnum


class CuttingMode(IntEnum):
    """job_mode"""

    single_grid = 0
    double_grid = 1
    segment_grid = 2
    no_grid = 3


class BorderPatrolMode(IntEnum):
    """"""

    none = 0
    one = 1
    two = 2
    three = 3
    four = 4


class ObstacleLapsMode(IntEnum):
    """mowingLaps"""

    none = 0
    one = 1
    two = 2
    three = 3
    four = 4


class MowOrder(IntEnum):
    """path_order"""

    border_first = 0
    grid_first = 1


class TraversalMode(IntEnum):
    """Traversal mode when returning."""

    direct = 0
    follow_perimeter = 1


class TurningMode(IntEnum):
    """Turning mode on corners."""

    zero_turn = 0
    multipoint = 1


class DetectionStrategy(IntEnum):
    """Matches up with ultra_wave."""

    direct_touch = 0
    slow_touch = 1
    less_touch = 2
    no_touch = 10  # luba 2 yuka only or possibly value of 10
    sensitive = 11  # x series


class PathAngleSetting(IntEnum):
    """Path Angle type."""

    relative_angle = 0
    absolute_angle = 1
    random_angle = 2  # Luba Pro / Luba 2 Yuka only



"""Manage state from notifications into MowingDevice."""

from collections.abc import Awaitable, Callable
from datetime import UTC, datetime
import logging
from typing import Any

import betterproto

from pymammotion.data.model.device import MowingDevice
from pymammotion.data.model.device_info import SideLight
from pymammotion.data.model.enums import ConnectionPreference
from pymammotion.data.model.hash_list import AreaHashNameList, NavGetCommData, NavGetHashListData, Plan, SvgMessage
from pymammotion.data.model.work import CurrentTaskSettings
from pymammotion.data.mqtt.properties import ThingPropertiesMessage
from pymammotion.data.mqtt.status import ThingStatusMessage
from pymammotion.event.event import DataEvent
from pymammotion.proto import (
    AppGetAllAreaHashName,
    DeviceFwInfo,
    DeviceProductTypeInfoT,
    DrvDevInfoResp,
    DrvDevInfoResult,
    GetNetworkInfoRsp,
    LubaMsg,
    NavGetCommDataAck,
    NavGetHashListAck,
    NavPlanJobSet,
    NavReqCoverPath,
    NavSysParamMsg,
    NavUnableTimeSet,
    SvgMessageAckT,
    TimeCtrlLight,
    WifiIotStatusReport,
)

logger = logging.getLogger(__name__)


class StateManager:
    """Manage state."""

    def __init__(self, device: MowingDevice) -> None:
        self._device: MowingDevice = device
        self.last_updated_at = datetime.now(UTC)
        self.preference = ConnectionPreference.WIFI
        self.cloud_gethash_ack_callback: Callable[[NavGetHashListAck], Awaitable[None]] | None = None
        self.cloud_get_commondata_ack_callback: (
            Callable[[NavGetCommDataAck | SvgMessageAckT], Awaitable[None]] | None
        ) = None
        self.cloud_on_notification_callback = DataEvent()
        self.cloud_queue_command_callback = DataEvent()

        self.cloud_get_plan_callback: Callable[[NavPlanJobSet], Awaitable[None]] | None = None
        self.ble_gethash_ack_callback: Callable[[NavGetHashListAck], Awaitable[None]] | None = None
        self.ble_get_commondata_ack_callback: Callable[[NavGetCommDataAck | SvgMessageAckT], Awaitable[None]] | None = (
            None
        )
        self.ble_get_plan_callback: Callable[[NavPlanJobSet], Awaitable[None]] | None = None
        self.ble_on_notification_callback = DataEvent()
        self.ble_queue_command_callback = DataEvent()

        self.properties_callback = DataEvent()
        self.status_callback = DataEvent()

    def get_device(self) -> MowingDevice:
        """Get device."""
        return self._device

    def set_device(self, device: MowingDevice) -> None:
        """Set device."""
        self._device = device

    async def properties(self, thing_properties: ThingPropertiesMessage) -> None:
        # TODO update device based off thing properties
        self._device.mqtt_properties = thing_properties
        await self.on_properties_callback(thing_properties)

    async def status(self, thing_status: ThingStatusMessage) -> None:
        if not self._device.online:
            self._device.online = True
        self._device.status_properties = thing_status
        if self._device.mower_state.product_key == "":
            self._device.mower_state.product_key = thing_status.params.productKey
        await self.on_status_callback(thing_status)

    @property
    def online(self) -> bool:
        return self._device.online

    @online.setter
    def online(self, value: bool) -> None:
        self._device.online = value

    async def gethash_ack_callback(self, msg: NavGetHashListAck) -> None:
        if self.cloud_gethash_ack_callback:
            await self.cloud_gethash_ack_callback(msg)
        elif self.ble_gethash_ack_callback:
            await self.ble_gethash_ack_callback(msg)

    async def on_notification_callback(self, res: tuple[str, Any | None]) -> None:
        if self.cloud_on_notification_callback:
            await self.cloud_on_notification_callback.data_event(res)
        elif self.ble_on_notification_callback:
            await self.ble_on_notification_callback.data_event(res)

    async def on_properties_callback(self, thing_properties: ThingPropertiesMessage) -> None:
        """Call properties callback if it exists."""
        if self.properties_callback:
            await self.properties_callback.data_event(thing_properties)

    async def on_status_callback(self, thing_status: ThingStatusMessage) -> None:
        """Execute the status callback if it is set."""
        if self.status_callback:
            await self.status_callback.data_event(thing_status)

    async def get_commondata_ack_callback(self, comm_data: NavGetCommDataAck | SvgMessageAckT) -> None:
        """Asynchronously calls the appropriate callback based on available handlers."""
        if self.cloud_get_commondata_ack_callback:
            await self.cloud_get_commondata_ack_callback(comm_data)
        elif self.ble_get_commondata_ack_callback:
            await self.ble_get_commondata_ack_callback(comm_data)

    async def get_plan_callback(self, planjob: NavPlanJobSet) -> None:
        if self.cloud_get_plan_callback:
            await self.cloud_get_plan_callback(planjob)
        elif self.ble_get_plan_callback:
            await self.ble_get_plan_callback(planjob)

    async def notification(self, message: LubaMsg) -> None:
        """Handle protobuf notifications."""
        res = betterproto.which_one_of(message, "LubaSubMsg")
        self.last_updated_at = datetime.now(UTC)
        # additional catch all if we don't get a status update
        if not self._device.online:
            self._device.online = True

        match res[0]:
            case "nav":
                await self._update_nav_data(message)
            case "sys":
                self._update_sys_data(message)
            case "driver":
                self._update_driver_data(message)
            case "net":
                self._update_net_data(message)
            case "mul":
                self._update_mul_data(message)
            case "ota":
                self._update_ota_data(message)

        await self.on_notification_callback(res)

    async def _update_nav_data(self, message) -> None:
        """Update nav data."""
        nav_msg = betterproto.which_one_of(message.nav, "SubNavMsg")
        match nav_msg[0]:
            case "toapp_gethash_ack":
                hashlist_ack: NavGetHashListAck = nav_msg[1]
                self._device.map.update_root_hash_list(
                    NavGetHashListData.from_dict(hashlist_ack.to_dict(casing=betterproto.Casing.SNAKE))
                )
                await self.gethash_ack_callback(nav_msg[1])
            case "toapp_get_commondata_ack":
                common_data: NavGetCommDataAck = nav_msg[1]
                updated = self._device.map.update(
                    NavGetCommData.from_dict(common_data.to_dict(casing=betterproto.Casing.SNAKE))
                )
                if updated:
                    await self.get_commondata_ack_callback(common_data)
            case "todev_planjob_set":
                planjob: NavPlanJobSet = nav_msg[1]
                self._device.map.update_plan(Plan.from_dict(planjob.to_dict(casing=betterproto.Casing.SNAKE)))
                await self.get_plan_callback(planjob)

            case "toapp_svg_msg":
                common_svg_data: SvgMessageAckT = nav_msg[1]
                updated = self._device.map.update(
                    SvgMessage.from_dict(common_svg_data.to_dict(casing=betterproto.Casing.SNAKE))
                )
                if updated:
                    await self.get_commondata_ack_callback(common_svg_data)

            case "toapp_all_hash_name":
                hash_names: AppGetAllAreaHashName = nav_msg[1]
                converted_list = [AreaHashNameList(name=item.name, hash=item.hash) for item in hash_names.hashnames]
                self._device.map.area_name = converted_list

            case "bidire_reqconver_path":
                work_settings: NavReqCoverPath = nav_msg[1]
                self._device.work = CurrentTaskSettings.from_dict(
                    work_settings.to_dict(casing=betterproto.Casing.SNAKE)
                )
            case "nav_sys_param_cmd":
                settings: NavSysParamMsg = nav_msg[1]
                match settings.id:
                    case 3:
                        self._device.mower_state.rain_detection = bool(settings.context)
                    case 6:
                        self._device.mower_state.turning_mode = settings.context
                    case 7:
                        self._device.mower_state.traversal_mode = settings.context
            case "todev_unable_time_set":
                nav_non_work_time: NavUnableTimeSet = nav_msg[1]
                self._device.non_work_hours.start_time = nav_non_work_time.unable_start_time
                self._device.non_work_hours.end_time = nav_non_work_time.unable_end_time

    def _update_sys_data(self, message) -> None:
        """Update system."""
        sys_msg = betterproto.which_one_of(message.sys, "SubSysMsg")
        match sys_msg[0]:
            case "system_update_buf":
                self._device.buffer(sys_msg[1])
            case "toapp_report_data":
                self._device.update_report_data(sys_msg[1])
            case "mow_to_app_info":
                self._device.mow_info(sys_msg[1])
            case "system_tard_state_tunnel":
                self._device.run_state_update(sys_msg[1])
            case "todev_time_ctrl_light":
                ctrl_light: TimeCtrlLight = sys_msg[1]
                side_led: SideLight = SideLight.from_dict(ctrl_light.to_dict(casing=betterproto.Casing.SNAKE))
                self._device.mower_state.side_led = side_led
            case "device_product_type_info":
                device_product_type: DeviceProductTypeInfoT = sys_msg[1]
                if device_product_type.main_product_type != "" or device_product_type.sub_product_type != "":
                    self._device.mower_state.model_id = device_product_type.main_product_type
                    self._device.mower_state.sub_model_id = device_product_type.sub_product_type
            case "toapp_dev_fw_info":
                device_fw_info: DeviceFwInfo = sys_msg[1]
                self._device.device_firmwares.device_version = device_fw_info.version
                self._device.mower_state.swversion = device_fw_info.version

    def _update_driver_data(self, message) -> None:
        pass

    def _update_net_data(self, message) -> None:
        net_msg = betterproto.which_one_of(message.net, "NetSubType")
        match net_msg[0]:
            case "toapp_wifi_iot_status":
                wifi_iot_status: WifiIotStatusReport = net_msg[1]
                self._device.mower_state.product_key = wifi_iot_status.productkey
            case "toapp_devinfo_resp":
                toapp_devinfo_resp: DrvDevInfoResp = net_msg[1]
                for resp in toapp_devinfo_resp.resp_ids:
                    if resp.res == DrvDevInfoResult.DRV_RESULT_SUC and resp.id == 1 and resp.type == 6:
                        self._device.mower_state.swversion = resp.info
                        self._device.device_firmwares.device_version = resp.info
            case "toapp_networkinfo_rsp":
                get_network_info_resp: GetNetworkInfoRsp = net_msg[1]
                self._device.mower_state.wifi_mac = get_network_info_resp.wifi_mac

    def _update_mul_data(self, message) -> None:
        pass

    def _update_ota_data(self, message) -> None:
        pass



from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

COLLECT_ABNORMAL: CollectMotorState
COLLECT_CLOSE: CollectMotorState
COLLECT_OPEN: CollectMotorState
COLLECT_STUCK: CollectMotorState
CUTTER_ECONOMIC: CutterWorkMode
CUTTER_PERFORMANCE: CutterWorkMode
CUTTER_STANDARD: CutterWorkMode
DESCRIPTOR: _descriptor.FileDescriptor
UNLOAD_CLOSE: UnloadMotorState
UNLOAD_OPEN: UnloadMotorState
UNLOAD_RUNNING: UnloadMotorState
UNLOAD_STOP: UnloadMotorState

class AppGetCutterWorkMode(_message.Message):
    __slots__ = ["QueryResult", "current_cutter_mode", "current_cutter_rpm"]
    CURRENT_CUTTER_MODE_FIELD_NUMBER: _ClassVar[int]
    CURRENT_CUTTER_RPM_FIELD_NUMBER: _ClassVar[int]
    QUERYRESULT_FIELD_NUMBER: _ClassVar[int]
    QueryResult: int
    current_cutter_mode: int
    current_cutter_rpm: int
    def __init__(self, current_cutter_mode: _Optional[int] = ..., current_cutter_rpm: _Optional[int] = ..., QueryResult: _Optional[int] = ...) -> None: ...

class AppSetCutterWorkMode(_message.Message):
    __slots__ = ["CutterMode", "SetResult"]
    CUTTERMODE_FIELD_NUMBER: _ClassVar[int]
    CutterMode: int
    SETRESULT_FIELD_NUMBER: _ClassVar[int]
    SetResult: int
    def __init__(self, CutterMode: _Optional[int] = ..., SetResult: _Optional[int] = ...) -> None: ...

class DrvCollectCtrlByHand(_message.Message):
    __slots__ = ["collect_ctrl", "unload_ctrl"]
    COLLECT_CTRL_FIELD_NUMBER: _ClassVar[int]
    UNLOAD_CTRL_FIELD_NUMBER: _ClassVar[int]
    collect_ctrl: int
    unload_ctrl: int
    def __init__(self, collect_ctrl: _Optional[int] = ..., unload_ctrl: _Optional[int] = ...) -> None: ...

class DrvKnifeChangeReport(_message.Message):
    __slots__ = ["cur_height", "end_height", "is_start", "start_height"]
    CUR_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    END_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    IS_START_FIELD_NUMBER: _ClassVar[int]
    START_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    cur_height: int
    end_height: int
    is_start: int
    start_height: int
    def __init__(self, is_start: _Optional[int] = ..., start_height: _Optional[int] = ..., end_height: _Optional[int] = ..., cur_height: _Optional[int] = ...) -> None: ...

class DrvKnifeHeight(_message.Message):
    __slots__ = ["knifeHeight"]
    KNIFEHEIGHT_FIELD_NUMBER: _ClassVar[int]
    knifeHeight: int
    def __init__(self, knifeHeight: _Optional[int] = ...) -> None: ...

class DrvKnifeStatus(_message.Message):
    __slots__ = ["knife_status"]
    KNIFE_STATUS_FIELD_NUMBER: _ClassVar[int]
    knife_status: int
    def __init__(self, knife_status: _Optional[int] = ...) -> None: ...

class DrvMotionCtrl(_message.Message):
    __slots__ = ["setAngularSpeed", "setLinearSpeed"]
    SETANGULARSPEED_FIELD_NUMBER: _ClassVar[int]
    SETLINEARSPEED_FIELD_NUMBER: _ClassVar[int]
    setAngularSpeed: int
    setLinearSpeed: int
    def __init__(self, setLinearSpeed: _Optional[int] = ..., setAngularSpeed: _Optional[int] = ...) -> None: ...

class DrvMowCtrlByHand(_message.Message):
    __slots__ = ["cut_knife_ctrl", "cut_knife_height", "main_ctrl", "max_run_Speed"]
    CUT_KNIFE_CTRL_FIELD_NUMBER: _ClassVar[int]
    CUT_KNIFE_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    MAIN_CTRL_FIELD_NUMBER: _ClassVar[int]
    MAX_RUN_SPEED_FIELD_NUMBER: _ClassVar[int]
    cut_knife_ctrl: int
    cut_knife_height: int
    main_ctrl: int
    max_run_Speed: float
    def __init__(self, main_ctrl: _Optional[int] = ..., cut_knife_ctrl: _Optional[int] = ..., cut_knife_height: _Optional[int] = ..., max_run_Speed: _Optional[float] = ...) -> None: ...

class DrvSrSpeed(_message.Message):
    __slots__ = ["rw", "speed"]
    RW_FIELD_NUMBER: _ClassVar[int]
    SPEED_FIELD_NUMBER: _ClassVar[int]
    rw: int
    speed: float
    def __init__(self, rw: _Optional[int] = ..., speed: _Optional[float] = ...) -> None: ...

class MctlDriver(_message.Message):
    __slots__ = ["bidire_knife_height_report", "bidire_speed_read_set", "collect_ctrl_by_hand", "current_cutter_mode", "cutter_mode_ctrl_by_hand", "mow_ctrl_by_hand", "rtk_cfg_req", "rtk_cfg_req_ack", "rtk_sys_mask_query", "rtk_sys_mask_query_ack", "toapp_knife_status", "toapp_knife_status_change", "todev_devmotion_ctrl", "todev_knife_height_set"]
    BIDIRE_KNIFE_HEIGHT_REPORT_FIELD_NUMBER: _ClassVar[int]
    BIDIRE_SPEED_READ_SET_FIELD_NUMBER: _ClassVar[int]
    COLLECT_CTRL_BY_HAND_FIELD_NUMBER: _ClassVar[int]
    CURRENT_CUTTER_MODE_FIELD_NUMBER: _ClassVar[int]
    CUTTER_MODE_CTRL_BY_HAND_FIELD_NUMBER: _ClassVar[int]
    MOW_CTRL_BY_HAND_FIELD_NUMBER: _ClassVar[int]
    RTK_CFG_REQ_ACK_FIELD_NUMBER: _ClassVar[int]
    RTK_CFG_REQ_FIELD_NUMBER: _ClassVar[int]
    RTK_SYS_MASK_QUERY_ACK_FIELD_NUMBER: _ClassVar[int]
    RTK_SYS_MASK_QUERY_FIELD_NUMBER: _ClassVar[int]
    TOAPP_KNIFE_STATUS_CHANGE_FIELD_NUMBER: _ClassVar[int]
    TOAPP_KNIFE_STATUS_FIELD_NUMBER: _ClassVar[int]
    TODEV_DEVMOTION_CTRL_FIELD_NUMBER: _ClassVar[int]
    TODEV_KNIFE_HEIGHT_SET_FIELD_NUMBER: _ClassVar[int]
    bidire_knife_height_report: DrvKnifeHeight
    bidire_speed_read_set: DrvSrSpeed
    collect_ctrl_by_hand: DrvCollectCtrlByHand
    current_cutter_mode: AppGetCutterWorkMode
    cutter_mode_ctrl_by_hand: AppSetCutterWorkMode
    mow_ctrl_by_hand: DrvMowCtrlByHand
    rtk_cfg_req: rtk_cfg_req_t
    rtk_cfg_req_ack: rtk_cfg_req_ack_t
    rtk_sys_mask_query: rtk_sys_mask_query_t
    rtk_sys_mask_query_ack: rtk_sys_mask_query_ack_t
    toapp_knife_status: DrvKnifeStatus
    toapp_knife_status_change: DrvKnifeChangeReport
    todev_devmotion_ctrl: DrvMotionCtrl
    todev_knife_height_set: DrvKnifeHeight
    def __init__(self, todev_devmotion_ctrl: _Optional[_Union[DrvMotionCtrl, _Mapping]] = ..., todev_knife_height_set: _Optional[_Union[DrvKnifeHeight, _Mapping]] = ..., bidire_speed_read_set: _Optional[_Union[DrvSrSpeed, _Mapping]] = ..., bidire_knife_height_report: _Optional[_Union[DrvKnifeHeight, _Mapping]] = ..., toapp_knife_status: _Optional[_Union[DrvKnifeStatus, _Mapping]] = ..., mow_ctrl_by_hand: _Optional[_Union[DrvMowCtrlByHand, _Mapping]] = ..., rtk_cfg_req: _Optional[_Union[rtk_cfg_req_t, _Mapping]] = ..., rtk_cfg_req_ack: _Optional[_Union[rtk_cfg_req_ack_t, _Mapping]] = ..., rtk_sys_mask_query: _Optional[_Union[rtk_sys_mask_query_t, _Mapping]] = ..., rtk_sys_mask_query_ack: _Optional[_Union[rtk_sys_mask_query_ack_t, _Mapping]] = ..., toapp_knife_status_change: _Optional[_Union[DrvKnifeChangeReport, _Mapping]] = ..., collect_ctrl_by_hand: _Optional[_Union[DrvCollectCtrlByHand, _Mapping]] = ..., cutter_mode_ctrl_by_hand: _Optional[_Union[AppSetCutterWorkMode, _Mapping]] = ..., current_cutter_mode: _Optional[_Union[AppGetCutterWorkMode, _Mapping]] = ...) -> None: ...

class rtk_cfg_req_ack_t(_message.Message):
    __slots__ = ["cmd_length", "cmd_response"]
    CMD_LENGTH_FIELD_NUMBER: _ClassVar[int]
    CMD_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    cmd_length: int
    cmd_response: str
    def __init__(self, cmd_length: _Optional[int] = ..., cmd_response: _Optional[str] = ...) -> None: ...

class rtk_cfg_req_t(_message.Message):
    __slots__ = ["cmd_length", "cmd_req"]
    CMD_LENGTH_FIELD_NUMBER: _ClassVar[int]
    CMD_REQ_FIELD_NUMBER: _ClassVar[int]
    cmd_length: int
    cmd_req: str
    def __init__(self, cmd_length: _Optional[int] = ..., cmd_req: _Optional[str] = ...) -> None: ...

class rtk_sys_mask_query_ack_t(_message.Message):
    __slots__ = ["sat_system", "system_mask_bits"]
    SAT_SYSTEM_FIELD_NUMBER: _ClassVar[int]
    SYSTEM_MASK_BITS_FIELD_NUMBER: _ClassVar[int]
    sat_system: int
    system_mask_bits: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, sat_system: _Optional[int] = ..., system_mask_bits: _Optional[_Iterable[int]] = ...) -> None: ...

class rtk_sys_mask_query_t(_message.Message):
    __slots__ = ["sat_system"]
    SAT_SYSTEM_FIELD_NUMBER: _ClassVar[int]
    sat_system: int
    def __init__(self, sat_system: _Optional[int] = ...) -> None: ...

class CutterWorkMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class CollectMotorState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class UnloadMotorState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pymammotion/proto/dev_net.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fpymammotion/proto/dev_net.proto\"(\n\rDrvWifiUpload\x12\x17\n\x0fwifi_msg_upload\x18\x01 \x01(\x05\"&\n\x0b\x44rvWifiList\x12\x17\n\x0fnvs_wifi_upload\x18\x01 \x01(\x05\"H\n\nDrvWifiSet\x12\x13\n\x0b\x63onfigParam\x18\x01 \x01(\x05\x12\x10\n\x08\x43onfssid\x18\x02 \x01(\t\x12\x13\n\x0bwifi_enable\x18\x03 \x01(\x08\"\xa8\x01\n\nDrvWifiMsg\x12\x0f\n\x07status1\x18\x01 \x01(\x08\x12\x0f\n\x07status2\x18\x02 \x01(\x08\x12\n\n\x02ip\x18\x03 \x01(\t\x12\x0f\n\x07msgssid\x18\x04 \x01(\t\x12\x10\n\x08password\x18\x05 \x01(\t\x12\x0c\n\x04rssi\x18\x06 \x01(\x05\x12\x12\n\nproductkey\x18\x07 \x01(\t\x12\x12\n\ndevicename\x18\x08 \x01(\t\x12\x13\n\x0bwifi_enable\x18\t \x01(\x08\"?\n\x0b\x44rvWifiConf\x12\x10\n\x08succFlag\x18\x01 \x01(\x08\x12\x0c\n\x04\x63ode\x18\x02 \x01(\x05\x12\x10\n\x08\x43onfssid\x18\x03 \x01(\t\"\\\n\rDrvListUpload\x12\x0b\n\x03sum\x18\x01 \x01(\x05\x12\x0f\n\x07\x63urrent\x18\x02 \x01(\x05\x12\x0e\n\x06status\x18\x03 \x01(\x05\x12\x0f\n\x07Memssid\x18\x04 \x01(\t\x12\x0c\n\x04rssi\x18\x05 \x01(\x05\"Y\n\x10\x44rvUploadFileReq\x12\r\n\x05\x62izId\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\x12\x0e\n\x06userId\x18\x03 \x01(\t\x12\x0b\n\x03num\x18\x04 \x01(\x05\x12\x0c\n\x04type\x18\x05 \x01(\x05\"$\n\x13\x44rvUploadFileCancel\x12\r\n\x05\x62izId\x18\x01 \x01(\t\"z\n\x15\x44rvUploadFileToAppReq\x12\r\n\x05\x62izId\x18\x01 \x01(\t\x12\x11\n\toperation\x18\x02 \x01(\x05\x12\x10\n\x08serverIp\x18\x03 \x01(\x07\x12\x12\n\nserverPort\x18\x04 \x01(\x05\x12\x0b\n\x03num\x18\x05 \x01(\x05\x12\x0c\n\x04type\x18\x06 \x01(\x05\"I\n\x15\x44rvUploadFileToAppRsp\x12\r\n\x05\x62izId\x18\x01 \x01(\t\x12\x11\n\toperation\x18\x02 \x01(\x05\x12\x0e\n\x06result\x18\x03 \x01(\x05\"+\n\x0f\x44rvDevInfoReqId\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04type\x18\x02 \x01(\x05\"Z\n\x10\x44rvDevInfoRespId\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04type\x18\x02 \x01(\x05\x12\x1e\n\x03res\x18\x03 \x01(\x0e\x32\x11.DrvDevInfoResult\x12\x0c\n\x04info\x18\x04 \x01(\t\"2\n\rDrvDevInfoReq\x12!\n\x07req_ids\x18\x01 \x03(\x0b\x32\x10.DrvDevInfoReqId\"5\n\x0e\x44rvDevInfoResp\x12#\n\x08resp_ids\x18\x01 \x03(\x0b\x32\x11.DrvDevInfoRespId\"\x9c\x01\n\x10\x44rvUpgradeReport\x12\x0f\n\x07\x64\x65vname\x18\x01 \x01(\t\x12\r\n\x05otaid\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\x12\x10\n\x08progress\x18\x04 \x01(\x05\x12\x0e\n\x06result\x18\x05 \x01(\x05\x12\x0f\n\x07message\x18\x06 \x01(\t\x12\x12\n\nproperties\x18\x07 \x01(\t\x12\x10\n\x08recv_cnt\x18\x08 \x01(\x05\"l\n\x13WifiIotStatusReport\x12\x16\n\x0ewifi_connected\x18\x01 \x01(\x08\x12\x15\n\riot_connected\x18\x02 \x01(\x08\x12\x12\n\nproductkey\x18\x03 \x01(\t\x12\x12\n\ndevicename\x18\x04 \x01(\t\"*\n\x0c\x42leTestBytes\x12\x0c\n\x04seqs\x18\x01 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x02 \x03(\x07\"$\n\x11GetNetworkInfoReq\x12\x0f\n\x07req_ids\x18\x01 \x01(\x05\"\x87\x01\n\x11GetNetworkInfoRsp\x12\x0f\n\x07req_ids\x18\x01 \x01(\x05\x12\x11\n\twifi_ssid\x18\x02 \x01(\t\x12\x10\n\x08wifi_mac\x18\x03 \x01(\t\x12\x11\n\twifi_rssi\x18\x04 \x01(\x05\x12\n\n\x02ip\x18\x05 \x01(\x07\x12\x0c\n\x04mask\x18\x06 \x01(\x07\x12\x0f\n\x07gateway\x18\x07 \x01(\x07\"N\n\x10mnet_inet_status\x12\x0f\n\x07\x63onnect\x18\x01 \x01(\x08\x12\n\n\x02ip\x18\x02 \x01(\x07\x12\x0c\n\x04mask\x18\x03 \x01(\x07\x12\x0f\n\x07gateway\x18\x04 \x01(\x07\"\xd7\x01\n\x08MnetInfo\x12\r\n\x05model\x18\x01 \x01(\t\x12\x10\n\x08revision\x18\x02 \x01(\t\x12\x0c\n\x04imei\x18\x03 \x01(\t\x12\x1a\n\x03sim\x18\x04 \x01(\x0e\x32\r.sim_card_sta\x12\x0c\n\x04imsi\x18\x05 \x01(\t\x12\"\n\tlink_type\x18\x06 \x01(\x0e\x32\x0f.mnet_link_type\x12\x0c\n\x04rssi\x18\x07 \x01(\x05\x12\x1f\n\x04inet\x18\x08 \x01(\x0b\x32\x11.mnet_inet_status\x12\r\n\x05iccid\x18\t \x01(\t\x12\x10\n\x08operator\x18\n \x01(\t\"!\n\x0eGetMnetInfoReq\x12\x0f\n\x07req_ids\x18\x01 \x01(\x05\"J\n\x0eGetMnetInfoRsp\x12\x0f\n\x07req_ids\x18\x01 \x01(\x05\x12\x0e\n\x06result\x18\x02 \x01(\x05\x12\x17\n\x04mnet\x18\x03 \x01(\x0b\x32\t.MnetInfo\"}\n\x07MnetApn\x12\x0b\n\x03\x63id\x18\x01 \x01(\x05\x12\x11\n\tapn_alias\x18\x02 \x01(\t\x12\x10\n\x08\x61pn_name\x18\x03 \x01(\t\x12\x1c\n\x04\x61uth\x18\x04 \x01(\x0e\x32\x0e.apn_auth_type\x12\x10\n\x08username\x18\x05 \x01(\t\x12\x10\n\x08password\x18\x06 \x01(\t\"9\n\nMnetApnCfg\x12\x14\n\x0c\x61pn_used_idx\x18\x01 \x01(\x05\x12\x15\n\x03\x61pn\x18\x02 \x03(\x0b\x32\x08.MnetApn\">\n\rMnetApnSetCfg\x12\x13\n\x0buse_default\x18\x01 \x01(\x08\x12\x18\n\x03\x63\x66g\x18\x02 \x01(\x0b\x32\x0b.MnetApnCfg\"~\n\x07MnetCfg\x12\x13\n\x0bmnet_enable\x18\x01 \x01(\x08\x12\x13\n\x0binet_enable\x18\x02 \x01(\x08\x12\x17\n\x04type\x18\x03 \x01(\x0e\x32\t.net_type\x12\x1b\n\x03\x61pn\x18\x04 \x01(\x0b\x32\x0e.MnetApnSetCfg\x12\x13\n\x0b\x61uto_select\x18\x05 \x01(\x08\" \n\rGetMnetCfgReq\x12\x0f\n\x07req_ids\x18\x01 \x01(\x05\"G\n\rGetMnetCfgRsp\x12\x0f\n\x07req_ids\x18\x01 \x01(\x05\x12\x0e\n\x06result\x18\x02 \x01(\x05\x12\x15\n\x03\x63\x66g\x18\x03 \x01(\x0b\x32\x08.MnetCfg\"7\n\rSetMnetCfgReq\x12\x0f\n\x07req_ids\x18\x01 \x01(\x05\x12\x15\n\x03\x63\x66g\x18\x02 \x01(\x0b\x32\x08.MnetCfg\"0\n\rSetMnetCfgRsp\x12\x0f\n\x07req_ids\x18\x01 \x01(\x05\x12\x0e\n\x06result\x18\x02 \x01(\x05\"N\n\x0e\x44rvDebugDdsZmq\x12\x11\n\tis_enable\x18\x01 \x01(\x08\x12\x15\n\rrx_topic_name\x18\x02 \x01(\t\x12\x12\n\ntx_zmq_url\x18\x03 \x01(\t\"!\n\x0cSetDrvBleMTU\x12\x11\n\tmtu_count\x18\x01 \x01(\x05\">\n\x0f\x42leSignatureReq\x12\x16\n\x0esignature_data\x18\x01 \x01(\t\x12\x13\n\x0brandom_data\x18\x02 \x01(\t\"\xb0\x01\n\x1a\x42leLogUploadUpdateProgress\x12\x0b\n\x03ver\x18\x01 \x01(\x05\x12\x0e\n\x06status\x18\x02 \x01(\x05\x12\r\n\x05\x62izId\x18\x03 \x01(\t\x12\x0c\n\x04prog\x18\x04 \x01(\x05\x12\x0f\n\x07subProg\x18\x05 \x01(\x05\x12\x0c\n\x04\x66Num\x18\x06 \x01(\x05\x12\r\n\x05speed\x18\x07 \x01(\x05\x12\x0c\n\x04\x66Idx\x18\x08 \x01(\x05\x12\r\n\x05\x66Path\x18\t \x01(\t\x12\r\n\x05\x66Size\x18\n \x01(\x05\"\xcb\x0b\n\x06\x44\x65vNet\x12\x18\n\x0etodev_ble_sync\x18\x01 \x01(\x05H\x00\x12\'\n\x0etodev_ConfType\x18\x02 \x01(\x0e\x32\r.WifiConfTypeH\x00\x12-\n\x13todev_WifiMsgUpload\x18\x03 \x01(\x0b\x32\x0e.DrvWifiUploadH\x00\x12,\n\x14todev_WifiListUpload\x18\x04 \x01(\x0b\x32\x0c.DrvWifiListH\x00\x12/\n\x18todev_Wifi_Configuration\x18\x05 \x01(\x0b\x32\x0b.DrvWifiSetH\x00\x12$\n\rtoapp_WifiMsg\x18\x06 \x01(\x0b\x32\x0b.DrvWifiMsgH\x00\x12&\n\x0etoapp_WifiConf\x18\x07 \x01(\x0b\x32\x0c.DrvWifiConfH\x00\x12*\n\x10toapp_ListUpload\x18\x08 \x01(\x0b\x32\x0e.DrvListUploadH\x00\x12/\n\x12todev_req_log_info\x18\t \x01(\x0b\x32\x11.DrvUploadFileReqH\x00\x12\x35\n\x15todev_log_data_cancel\x18\n \x01(\x0b\x32\x14.DrvUploadFileCancelH\x00\x12+\n\x11todev_devinfo_req\x18\x0b \x01(\x0b\x32\x0e.DrvDevInfoReqH\x00\x12-\n\x12toapp_devinfo_resp\x18\x0c \x01(\x0b\x32\x0f.DrvDevInfoRespH\x00\x12\x31\n\x14toapp_upgrade_report\x18\r \x01(\x0b\x32\x11.DrvUpgradeReportH\x00\x12\x35\n\x15toapp_wifi_iot_status\x18\x0e \x01(\x0b\x32\x14.WifiIotStatusReportH\x00\x12\x36\n\x14todev_uploadfile_req\x18\x0f \x01(\x0b\x32\x16.DrvUploadFileToAppReqH\x00\x12\x36\n\x14toapp_uploadfile_rsp\x18\x10 \x01(\x0b\x32\x16.DrvUploadFileToAppRspH\x00\x12\x33\n\x15todev_networkinfo_req\x18\x11 \x01(\x0b\x32\x12.GetNetworkInfoReqH\x00\x12\x33\n\x15toapp_networkinfo_rsp\x18\x12 \x01(\x0b\x32\x12.GetNetworkInfoRspH\x00\x12%\n\x0c\x62ir_testdata\x18\x13 \x01(\x0b\x32\r.BleTestBytesH\x00\x12.\n\x13todev_mnet_info_req\x18\x14 \x01(\x0b\x32\x0f.GetMnetInfoReqH\x00\x12.\n\x13toapp_mnet_info_rsp\x18\x15 \x01(\x0b\x32\x0f.GetMnetInfoRspH\x00\x12\x30\n\x16todev_get_mnet_cfg_req\x18\x16 \x01(\x0b\x32\x0e.GetMnetCfgReqH\x00\x12\x30\n\x16toapp_get_mnet_cfg_rsp\x18\x17 \x01(\x0b\x32\x0e.GetMnetCfgRspH\x00\x12\x30\n\x16todev_set_mnet_cfg_req\x18\x18 \x01(\x0b\x32\x0e.SetMnetCfgReqH\x00\x12\x30\n\x16toapp_set_mnet_cfg_rsp\x18\x19 \x01(\x0b\x32\x0e.SetMnetCfgRspH\x00\x12,\n\x11todev_set_dds2zmq\x18\x1a \x01(\x0b\x32\x0f.DrvDebugDdsZmqH\x00\x12*\n\x11todev_set_ble_mtu\x18\x1b \x01(\x0b\x32\r.SetDrvBleMTUH\x00\x12\x36\n\x19todev_set_iot_offline_req\x18\x1c \x01(\x0e\x32\x11.iot_conctrl_typeH\x00\x12\x36\n\x1atodev_verify_signature_req\x18\x1d \x01(\x0b\x32\x10.BleSignatureReqH\x00\x12G\n toapp_log_upload_update_progress\x18\x1e \x01(\x0b\x32\x1b.BleLogUploadUpdateProgressH\x00\x42\x0c\n\nNetSubType*l\n\x0cWifiConfType\x12\x12\n\x0e\x44isconnectWifi\x10\x00\x12\x0e\n\nForgetWifi\x10\x01\x12\x15\n\x11\x44irectConnectWifi\x10\x02\x12\x11\n\rReconnectWifi\x10\x03\x12\x0e\n\nset_enable\x10\x04*l\n\x15\x44rvUploadFileFileType\x12\x11\n\rFILE_TYPE_ALL\x10\x00\x12\x14\n\x10\x46ILE_TYPE_SYSLOG\x10\x01\x12\x14\n\x10\x46ILE_TYPE_NAVLOG\x10\x02\x12\x14\n\x10\x46ILE_TYPE_RTKLOG\x10\x03*R\n\x10\x44rvDevInfoResult\x12\x13\n\x0f\x44RV_RESULT_FAIL\x10\x00\x12\x12\n\x0e\x44RV_RESULT_SUC\x10\x01\x12\x15\n\x11\x44RV_RESULT_NOTSUP\x10\x02*p\n\x0csim_card_sta\x12\x0c\n\x08SIM_NONE\x10\x00\x12\x0f\n\x0bSIM_NO_CARD\x10\x01\x12\x0f\n\x0bSIM_INVALID\x10\x02\x12\x11\n\rSIM_INPUT_PIN\x10\x03\x12\x11\n\rSIM_INPUT_PUK\x10\x04\x12\n\n\x06SIM_OK\x10\x05*Z\n\x0emnet_link_type\x12\x12\n\x0eMNET_LINK_NONE\x10\x00\x12\x10\n\x0cMNET_LINK_2G\x10\x01\x12\x10\n\x0cMNET_LINK_3G\x10\x02\x12\x10\n\x0cMNET_LINK_4G\x10\x03*^\n\rapn_auth_type\x12\x11\n\rAPN_AUTH_NONE\x10\x00\x12\x10\n\x0c\x41PN_AUTH_PAP\x10\x01\x12\x11\n\rAPN_AUTH_CHAP\x10\x02\x12\x15\n\x11\x41PN_AUTH_PAP_CHAP\x10\x03*0\n\x08net_type\x12\x11\n\rNET_TYPE_WIFI\x10\x00\x12\x11\n\rNET_TYPE_MNET\x10\x01*Q\n\x10iot_conctrl_type\x12\x14\n\x10IOT_TYPE_OFFLINE\x10\x00\x12\x13\n\x0fIOT_TYPE_ONLINE\x10\x01\x12\x12\n\x0eIOT_TYPE_RESET\x10\x02\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'pymammotion.proto.dev_net_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _WIFICONFTYPE._serialized_start=4427
  _WIFICONFTYPE._serialized_end=4535
  _DRVUPLOADFILEFILETYPE._serialized_start=4537
  _DRVUPLOADFILEFILETYPE._serialized_end=4645
  _DRVDEVINFORESULT._serialized_start=4647
  _DRVDEVINFORESULT._serialized_end=4729
  _SIM_CARD_STA._serialized_start=4731
  _SIM_CARD_STA._serialized_end=4843
  _MNET_LINK_TYPE._serialized_start=4845
  _MNET_LINK_TYPE._serialized_end=4935
  _APN_AUTH_TYPE._serialized_start=4937
  _APN_AUTH_TYPE._serialized_end=5031
  _NET_TYPE._serialized_start=5033
  _NET_TYPE._serialized_end=5081
  _IOT_CONCTRL_TYPE._serialized_start=5083
  _IOT_CONCTRL_TYPE._serialized_end=5164
  _DRVWIFIUPLOAD._serialized_start=35
  _DRVWIFIUPLOAD._serialized_end=75
  _DRVWIFILIST._serialized_start=77
  _DRVWIFILIST._serialized_end=115
  _DRVWIFISET._serialized_start=117
  _DRVWIFISET._serialized_end=189
  _DRVWIFIMSG._serialized_start=192
  _DRVWIFIMSG._serialized_end=360
  _DRVWIFICONF._serialized_start=362
  _DRVWIFICONF._serialized_end=425
  _DRVLISTUPLOAD._serialized_start=427
  _DRVLISTUPLOAD._serialized_end=519
  _DRVUPLOADFILEREQ._serialized_start=521
  _DRVUPLOADFILEREQ._serialized_end=610
  _DRVUPLOADFILECANCEL._serialized_start=612
  _DRVUPLOADFILECANCEL._serialized_end=648
  _DRVUPLOADFILETOAPPREQ._serialized_start=650
  _DRVUPLOADFILETOAPPREQ._serialized_end=772
  _DRVUPLOADFILETOAPPRSP._serialized_start=774
  _DRVUPLOADFILETOAPPRSP._serialized_end=847
  _DRVDEVINFOREQID._serialized_start=849
  _DRVDEVINFOREQID._serialized_end=892
  _DRVDEVINFORESPID._serialized_start=894
  _DRVDEVINFORESPID._serialized_end=984
  _DRVDEVINFOREQ._serialized_start=986
  _DRVDEVINFOREQ._serialized_end=1036
  _DRVDEVINFORESP._serialized_start=1038
  _DRVDEVINFORESP._serialized_end=1091
  _DRVUPGRADEREPORT._serialized_start=1094
  _DRVUPGRADEREPORT._serialized_end=1250
  _WIFIIOTSTATUSREPORT._serialized_start=1252
  _WIFIIOTSTATUSREPORT._serialized_end=1360
  _BLETESTBYTES._serialized_start=1362
  _BLETESTBYTES._serialized_end=1404
  _GETNETWORKINFOREQ._serialized_start=1406
  _GETNETWORKINFOREQ._serialized_end=1442
  _GETNETWORKINFORSP._serialized_start=1445
  _GETNETWORKINFORSP._serialized_end=1580
  _MNET_INET_STATUS._serialized_start=1582
  _MNET_INET_STATUS._serialized_end=1660
  _MNETINFO._serialized_start=1663
  _MNETINFO._serialized_end=1878
  _GETMNETINFOREQ._serialized_start=1880
  _GETMNETINFOREQ._serialized_end=1913
  _GETMNETINFORSP._serialized_start=1915
  _GETMNETINFORSP._serialized_end=1989
  _MNETAPN._serialized_start=1991
  _MNETAPN._serialized_end=2116
  _MNETAPNCFG._serialized_start=2118
  _MNETAPNCFG._serialized_end=2175
  _MNETAPNSETCFG._serialized_start=2177
  _MNETAPNSETCFG._serialized_end=2239
  _MNETCFG._serialized_start=2241
  _MNETCFG._serialized_end=2367
  _GETMNETCFGREQ._serialized_start=2369
  _GETMNETCFGREQ._serialized_end=2401
  _GETMNETCFGRSP._serialized_start=2403
  _GETMNETCFGRSP._serialized_end=2474
  _SETMNETCFGREQ._serialized_start=2476
  _SETMNETCFGREQ._serialized_end=2531
  _SETMNETCFGRSP._serialized_start=2533
  _SETMNETCFGRSP._serialized_end=2581
  _DRVDEBUGDDSZMQ._serialized_start=2583
  _DRVDEBUGDDSZMQ._serialized_end=2661
  _SETDRVBLEMTU._serialized_start=2663
  _SETDRVBLEMTU._serialized_end=2696
  _BLESIGNATUREREQ._serialized_start=2698
  _BLESIGNATUREREQ._serialized_end=2760
  _BLELOGUPLOADUPDATEPROGRESS._serialized_start=2763
  _BLELOGUPLOADUPDATEPROGRESS._serialized_end=2939
  _DEVNET._serialized_start=2942
  _DEVNET._serialized_end=4425
# @@protoc_insertion_point(module_scope)



from datetime import UTC, datetime
import time

from Tea.exceptions import UnretryableException
from Tea.request import TeaRequest

from pymammotion.aliyun.tea.core import TeaCore

try:
    from typing import Dict
except ImportError:
    pass

from alibabacloud_apigateway_util.client import Client as APIGatewayUtilClient
from alibabacloud_tea_util.client import Client as UtilClient


class Client:
    """test"""

    _app_key = None  # type: str
    _app_secret = None  # type: str
    _protocol = None  # type: str
    _user_agent = None  # type: str
    _read_timeout = None  # type: int
    _connect_timeout = None  # type: int
    _http_proxy = None  # type: str
    _https_proxy = None  # type: str
    _no_proxy = None  # type: str
    _max_idle_conns = None  # type: int
    _domain = None  # type: str

    def __init__(self, config) -> None:
        self._domain = config.domain
        self._app_key = config.app_key
        self._app_secret = config.app_secret
        self._protocol = config.protocol
        self._read_timeout = config.read_timeout
        self._connect_timeout = config.connect_timeout
        self._http_proxy = config.http_proxy
        self._https_proxy = config.https_proxy
        self._no_proxy = config.no_proxy
        self._max_idle_conns = config.max_idle_conns

    def do_request(self, pathname, protocol, method, header, body, runtime):
        """Send request

        @type pathname: str
        @param pathname: the url path

        @type protocol: str
        @param protocol: http or https

        @type method: str
        @param method: example GET

        @type header: Dict[str, str]
        @param header: request header

        @type body: iot_api_gateway_models.IoTApiRequest
        @param body: the object of IoTApiRequest

        @type runtime: util_models.RuntimeOptions
        @param runtime: which controls some details of call api, such as retry times

        @rtype: TeaResponse
        @return: the response
        """
        body.validate()
        runtime.validate()
        _runtime = {
            "timeouted": "retry",
            "readTimeout": UtilClient.default_number(runtime.read_timeout, self._read_timeout),
            "connectTimeout": UtilClient.default_number(runtime.connect_timeout, self._connect_timeout),
            "httpProxy": UtilClient.default_string(runtime.http_proxy, self._http_proxy),
            "httpsProxy": UtilClient.default_string(runtime.https_proxy, self._https_proxy),
            "noProxy": UtilClient.default_string(runtime.no_proxy, self._no_proxy),
            "maxIdleConns": UtilClient.default_number(runtime.max_idle_conns, self._max_idle_conns),
            "retry": {
                "retryable": runtime.autoretry,
                "maxAttempts": UtilClient.default_number(runtime.max_attempts, 3),
            },
            "backoff": {
                "policy": UtilClient.default_string(runtime.backoff_policy, "no"),
                "period": UtilClient.default_number(runtime.backoff_period, 1),
            },
            "ignoreSSL": runtime.ignore_ssl,
        }
        _last_request = None
        _last_exception = None
        _now = time.time()
        _retry_times = 0
        while TeaCore.allow_retry(_runtime.get("retry"), _retry_times, _now):
            if _retry_times > 0:
                _backoff_time = TeaCore.get_backoff_time(_runtime.get("backoff"), _retry_times)
                if _backoff_time > 0:
                    TeaCore.sleep(_backoff_time)
            _retry_times = _retry_times + 1
            try:
                _request = TeaRequest()
                _request.protocol = UtilClient.default_string(self._protocol, protocol)
                _request.method = UtilClient.default_string(method, "POST")
                _request.pathname = pathname
                _request.headers = TeaCore.merge(
                    {
                        "host": self._domain,
                        "date": datetime.now(UTC).strftime("%a, %d %b %Y %H:%M:%S GMT"),
                        "x-ca-nonce": UtilClient.get_nonce(),
                        "x-ca-key": self._app_key,
                        "x-ca-signaturemethod": "HmacSHA256",
                        "accept": "application/json",
                        "user-agent": self.get_user_agent(),
                    },
                    header,
                )
                if UtilClient.empty(body.id):
                    body.id = UtilClient.get_nonce()
                if not UtilClient.is_unset(body):
                    _request.headers["content-type"] = "application/octet-stream"
                    _request.headers["content-md5"] = APIGatewayUtilClient.get_content_md5(
                        UtilClient.to_jsonstring(TeaCore.to_map(body))
                    )
                    _request.body = UtilClient.to_jsonstring(TeaCore.to_map(body))
                _request.headers["x-ca-signature"] = APIGatewayUtilClient.get_signature(_request, self._app_secret)
                _last_request = _request
                _response = TeaCore.do_action(_request, _runtime)
                return _response
            except Exception as e:
                if TeaCore.is_retryable(e):
                    _last_exception = e
                    continue
                raise e
        raise UnretryableException(_last_request, _last_exception)

    async def async_do_request(self, pathname, protocol, method, header, body, runtime):
        """Send request

        @type pathname: str
        @param pathname: the url path

        @type protocol: str
        @param protocol: http or https

        @type method: str
        @param method: example GET

        @type header: Dict[str, str]
        @param header: request header

        @type body: iot_api_gateway_models.IoTApiRequest
        @param body: the object of IoTApiRequest

        @type runtime: util_models.RuntimeOptions
        @param runtime: which controls some details of call api, such as retry times

        @rtype: TeaResponse
        @return: the response
        """
        body.validate()
        runtime.validate()
        _runtime = {
            "timeouted": "retry",
            "readTimeout": UtilClient.default_number(runtime.read_timeout, self._read_timeout),
            "connectTimeout": UtilClient.default_number(runtime.connect_timeout, self._connect_timeout),
            "httpProxy": UtilClient.default_string(runtime.http_proxy, self._http_proxy),
            "httpsProxy": UtilClient.default_string(runtime.https_proxy, self._https_proxy),
            "noProxy": UtilClient.default_string(runtime.no_proxy, self._no_proxy),
            "maxIdleConns": UtilClient.default_number(runtime.max_idle_conns, self._max_idle_conns),
            "retry": {
                "retryable": runtime.autoretry,
                "maxAttempts": UtilClient.default_number(runtime.max_attempts, 3),
            },
            "backoff": {
                "policy": UtilClient.default_string(runtime.backoff_policy, "no"),
                "period": UtilClient.default_number(runtime.backoff_period, 1),
            },
            "ignoreSSL": runtime.ignore_ssl,
        }
        _last_request = None
        _last_exception = None
        _now = time.time()
        _retry_times = 0
        while TeaCore.allow_retry(_runtime.get("retry"), _retry_times, _now):
            if _retry_times > 0:
                _backoff_time = TeaCore.get_backoff_time(_runtime.get("backoff"), _retry_times)
                if _backoff_time > 0:
                    await TeaCore.sleep_async(_backoff_time)
            _retry_times = _retry_times + 1
            try:
                _request = TeaRequest()
                _request.protocol = UtilClient.default_string(self._protocol, protocol)
                _request.method = UtilClient.default_string(method, "POST")
                _request.pathname = pathname
                _request.headers = TeaCore.merge(
                    {
                        "host": self._domain,
                        "date": UtilClient.get_date_utcstring(),
                        "x-ca-nonce": UtilClient.get_nonce(),
                        "x-ca-key": self._app_key,
                        "x-ca-signaturemethod": "HmacSHA256",
                        "accept": "application/json",
                        "user-agent": self.get_user_agent(),
                    },
                    header,
                )
                if UtilClient.empty(body.id):
                    body.id = UtilClient.get_nonce()
                if not UtilClient.is_unset(body):
                    _request.headers["content-type"] = "application/octet-stream"
                    _request.headers["content-md5"] = APIGatewayUtilClient.get_content_md5(
                        UtilClient.to_jsonstring(TeaCore.to_map(body))
                    )
                    _request.body = UtilClient.to_jsonstring(TeaCore.to_map(body))
                _request.headers["x-ca-signature"] = APIGatewayUtilClient.get_signature(_request, self._app_secret)
                _last_request = _request
                _response = await TeaCore.async_do_action(_request, _runtime)
                return _response
            except Exception as e:
                if TeaCore.is_retryable(e):
                    _last_exception = e
                    continue
                raise e
        raise UnretryableException(_last_request, _last_exception)

    def get_user_agent(self):
        """Get user agent

        @rtype: str
        @return: user agent
        """
        user_agent = UtilClient.get_user_agent(self._user_agent)
        return user_agent



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pymammotion/proto/luba_mul.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n pymammotion/proto/luba_mul.proto\"o\n\x0bMulSetAudio\x12\x13\n\tat_switch\x18\x01 \x01(\x05H\x00\x12$\n\x0b\x61u_language\x18\x02 \x01(\x0e\x32\r.MUL_LANGUAGEH\x00\x12\x17\n\x03sex\x18\x03 \x01(\x0e\x32\x08.MUL_SEXH\x00\x42\x0c\n\nAudioCfg_u\"H\n\x0bMulSetVideo\x12&\n\x08position\x18\x01 \x01(\x0e\x32\x14.MUL_CAMERA_POSITION\x12\x11\n\tvi_switch\x18\x02 \x01(\x05\";\n\x0eMulSetVideoAck\x12)\n\nerror_code\x18\x01 \x01(\x0e\x32\x15.MUL_VIDEO_ERROR_CODE\"[\n\x0bMulAudioCfg\x12\x11\n\tau_switch\x18\x01 \x01(\x05\x12\"\n\x0b\x61u_language\x18\x02 \x01(\x0e\x32\r.MUL_LANGUAGE\x12\x15\n\x03sex\x18\x03 \x01(\x0e\x32\x08.MUL_SEX\"\x1c\n\x0bMulSetWiper\x12\r\n\x05round\x18\x01 \x01(\x05\"\x1e\n\x0bGetHeadlamp\x12\x0f\n\x07get_ids\x18\x01 \x01(\x05\"\xba\x01\n\x0bSetHeadlamp\x12\x0f\n\x07set_ids\x18\x01 \x01(\x05\x12\x17\n\x0flamp_power_ctrl\x18\x02 \x01(\x05\x12!\n\tlamp_ctrl\x18\x03 \x01(\x0e\x32\x0e.lamp_ctrl_sta\x12\x18\n\x10\x63trl_lamp_bright\x18\x04 \x01(\x08\x12\x13\n\x0blamp_bright\x18\x05 \x01(\x05\x12/\n\x10lamp_manual_ctrl\x18\x06 \x01(\x0e\x32\x15.lamp_manual_ctrl_sta\"-\n\nSetlamprsp\x12\x0f\n\x07set_ids\x18\x01 \x01(\x05\x12\x0e\n\x06result\x18\x02 \x01(\x05\"\x96\x01\n\nGetlamprsp\x12\x0f\n\x07get_ids\x18\x01 \x01(\x05\x12\x0e\n\x06result\x18\x02 \x01(\x05\x12!\n\tlamp_ctrl\x18\x03 \x01(\x0e\x32\x0e.lamp_ctrl_sta\x12\x13\n\x0blamp_bright\x18\x04 \x01(\x05\x12/\n\x10lamp_manual_ctrl\x18\x05 \x01(\x0e\x32\x15.lamp_manual_ctrl_sta\";\n\x0eMulSetWiperAck\x12)\n\nerror_code\x18\x01 \x01(\x0e\x32\x15.MUL_WIPER_ERROR_CODE\"\x1e\n\x0cMulSetEncode\x12\x0e\n\x06\x65ncode\x18\x01 \x01(\x08\"\xa5\x03\n\x06SocMul\x12!\n\tset_audio\x18\x01 \x01(\x0b\x32\x0c.MulSetAudioH\x00\x12!\n\taudio_cfg\x18\x02 \x01(\x0b\x32\x0c.MulAudioCfgH\x00\x12!\n\tset_video\x18\x03 \x01(\x0b\x32\x0c.MulSetVideoH\x00\x12(\n\rset_video_ack\x18\x04 \x01(\x0b\x32\x0f.MulSetVideoAckH\x00\x12!\n\tset_wiper\x18\x05 \x01(\x0b\x32\x0c.MulSetWiperH\x00\x12(\n\rset_wiper_ack\x18\x06 \x01(\x0b\x32\x0f.MulSetWiperAckH\x00\x12 \n\x08get_lamp\x18\x07 \x01(\x0b\x32\x0c.GetHeadlampH\x00\x12 \n\x08set_lamp\x18\x08 \x01(\x0b\x32\x0c.SetHeadlampH\x00\x12#\n\x0cset_lamp_rsp\x18\t \x01(\x0b\x32\x0b.SetlamprspH\x00\x12#\n\x0cget_lamp_rsp\x18\n \x01(\x0b\x32\x0b.GetlamprspH\x00\x12#\n\nreq_encode\x18\x0b \x01(\x0b\x32\r.MulSetEncodeH\x00\x42\x08\n\x06SubMul*\x83\x01\n\x0cMUL_LANGUAGE\x12\x0b\n\x07\x45NGLISH\x10\x00\x12\n\n\x06GERMAN\x10\x01\x12\n\n\x06\x46RENCH\x10\x02\x12\x0b\n\x07ITALIAN\x10\x03\x12\x0b\n\x07SPANISH\x10\x04\x12\x0e\n\nPORTUGUESE\x10\x05\x12\t\n\x05\x44UTCH\x10\x06\x12\x0b\n\x07SWEDISH\x10\x07\x12\x0c\n\x08NONE_LAN\x10\x08*+\n\x07MUL_SEX\x12\x07\n\x03MAN\x10\x00\x12\t\n\x05WOMAN\x10\x01\x12\x0c\n\x08NONE_SEX\x10\x02*=\n\x13MUL_CAMERA_POSITION\x12\x08\n\x04LEFT\x10\x00\x12\t\n\x05RIGHT\x10\x01\x12\x08\n\x04REAR\x10\x02\x12\x07\n\x03\x41LL\x10\x03*\x83\x01\n\x14MUL_VIDEO_ERROR_CODE\x12\x0b\n\x07SUCCESS\x10\x00\x12\x15\n\x11\x41\x43TIVATION_FAILED\x10\x01\x12\x19\n\x15NETWORK_NOT_AVAILABLE\x10\x02\x12\x19\n\x15\x43REATE_CHANNEL_FAILED\x10\x03\x12\x11\n\rPARAM_INVAILD\x10\x04*Q\n\x14MUL_WIPER_ERROR_CODE\x12\x0f\n\x0bSET_SUCCESS\x10\x00\x12\x0c\n\x08HW_ERROR\x10\x01\x12\x1a\n\x16NAVIGATION_WORK_FORBID\x10\x02*?\n\rlamp_ctrl_sta\x12\r\n\tpower_off\x10\x00\x12\x0c\n\x08power_on\x10\x01\x12\x11\n\rpower_ctrl_on\x10\x02*A\n\x14lamp_manual_ctrl_sta\x12\x14\n\x10manual_power_off\x10\x00\x12\x13\n\x0fmanual_power_on\x10\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'pymammotion.proto.luba_mul_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MUL_LANGUAGE._serialized_start=1346
  _MUL_LANGUAGE._serialized_end=1477
  _MUL_SEX._serialized_start=1479
  _MUL_SEX._serialized_end=1522
  _MUL_CAMERA_POSITION._serialized_start=1524
  _MUL_CAMERA_POSITION._serialized_end=1585
  _MUL_VIDEO_ERROR_CODE._serialized_start=1588
  _MUL_VIDEO_ERROR_CODE._serialized_end=1719
  _MUL_WIPER_ERROR_CODE._serialized_start=1721
  _MUL_WIPER_ERROR_CODE._serialized_end=1802
  _LAMP_CTRL_STA._serialized_start=1804
  _LAMP_CTRL_STA._serialized_end=1867
  _LAMP_MANUAL_CTRL_STA._serialized_start=1869
  _LAMP_MANUAL_CTRL_STA._serialized_end=1934
  _MULSETAUDIO._serialized_start=36
  _MULSETAUDIO._serialized_end=147
  _MULSETVIDEO._serialized_start=149
  _MULSETVIDEO._serialized_end=221
  _MULSETVIDEOACK._serialized_start=223
  _MULSETVIDEOACK._serialized_end=282
  _MULAUDIOCFG._serialized_start=284
  _MULAUDIOCFG._serialized_end=375
  _MULSETWIPER._serialized_start=377
  _MULSETWIPER._serialized_end=405
  _GETHEADLAMP._serialized_start=407
  _GETHEADLAMP._serialized_end=437
  _SETHEADLAMP._serialized_start=440
  _SETHEADLAMP._serialized_end=626
  _SETLAMPRSP._serialized_start=628
  _SETLAMPRSP._serialized_end=673
  _GETLAMPRSP._serialized_start=676
  _GETLAMPRSP._serialized_end=826
  _MULSETWIPERACK._serialized_start=828
  _MULSETWIPERACK._serialized_end=887
  _MULSETENCODE._serialized_start=889
  _MULSETENCODE._serialized_end=919
  _SOCMUL._serialized_start=922
  _SOCMUL._serialized_end=1343
# @@protoc_insertion_point(module_scope)



"""Device control of mammotion robots over bluetooth or MQTT."""

from __future__ import annotations

import asyncio
import logging
from typing import Any

from bleak.backends.device import BLEDevice

from pymammotion.aliyun.cloud_gateway import CloudIOTGateway
from pymammotion.aliyun.model.dev_by_account_response import Device
from pymammotion.data.model.device import MowingDevice
from pymammotion.data.model.enums import ConnectionPreference
from pymammotion.data.state_manager import StateManager
from pymammotion.http.http import MammotionHTTP
from pymammotion.http.model.camera_stream import StreamSubscriptionResponse, VideoResourceResponse
from pymammotion.http.model.http import Response
from pymammotion.mammotion.devices.mammotion_bluetooth import MammotionBaseBLEDevice
from pymammotion.mammotion.devices.mammotion_cloud import MammotionBaseCloudDevice, MammotionCloud
from pymammotion.mqtt import MammotionMQTT
from pymammotion.utility.device_type import DeviceType

TIMEOUT_CLOUD_RESPONSE = 10

_LOGGER = logging.getLogger(__name__)


class MammotionMixedDeviceManager:
    def __init__(
        self,
        name: str,
        iot_id: str,
        cloud_client: CloudIOTGateway,
        cloud_device: Device,
        ble_device: BLEDevice | None = None,
        mqtt: MammotionCloud | None = None,
        preference: ConnectionPreference = ConnectionPreference.BLUETOOTH,
    ) -> None:
        self._ble_device: MammotionBaseBLEDevice | None = None
        self._cloud_device: MammotionBaseCloudDevice | None = None
        self.name = name
        self.iot_id = iot_id
        self.cloud_client = cloud_client
        self._state_manager = StateManager(MowingDevice())
        self._state_manager.get_device().name = name
        self._device: Device = cloud_device
        self.add_ble(ble_device)
        self.add_cloud(mqtt)
        self.mammotion_http = cloud_client.mammotion_http
        self.preference = preference
        self._state_manager.preference = preference

    @property
    def state_manager(self) -> StateManager:
        """Return the state manager."""
        return self._state_manager

    @property
    def state(self):
        """Return the state of the device."""
        return self._state_manager.get_device()

    @state.setter
    def state(self, value: MowingDevice) -> None:
        self._state_manager.set_device(value)

    def ble(self) -> MammotionBaseBLEDevice | None:
        return self._ble_device

    def cloud(self) -> MammotionBaseCloudDevice | None:
        return self._cloud_device

    def has_queued_commands(self) -> bool:
        if self.has_cloud() and self.preference == ConnectionPreference.WIFI:
            return not self.cloud().mqtt.command_queue.empty()
        else:
            return not self.ble().command_queue.empty()

    def add_ble(self, ble_device: BLEDevice) -> None:
        if ble_device is not None:
            self._ble_device = MammotionBaseBLEDevice(
                state_manager=self._state_manager, cloud_device=self._device, device=ble_device
            )

    def add_cloud(self, mqtt: MammotionCloud) -> None:
        self._cloud_device = MammotionBaseCloudDevice(
            mqtt, cloud_device=self._device, state_manager=self._state_manager
        )

    def replace_cloud(self, cloud_device: MammotionBaseCloudDevice) -> None:
        self._cloud_device = cloud_device

    def remove_cloud(self) -> None:
        self._state_manager.cloud_get_commondata_ack_callback = None
        self._state_manager.cloud_get_hashlist_ack_callback = None
        self._state_manager.cloud_get_plan_callback = None
        self._state_manager.cloud_on_notification_callback = None
        self._state_manager.cloud_gethash_ack_callback = None
        self._cloud_device = None

    def replace_ble(self, ble_device: MammotionBaseBLEDevice) -> None:
        self._ble_device = ble_device

    def remove_ble(self) -> None:
        self._state_manager.ble_get_commondata_ack_callback = None
        self._state_manager.ble_get_hashlist_ack_callback = None
        self._state_manager.ble_get_plan_callback = None
        self._state_manager.ble_on_notification_callback = None
        self._state_manager.ble_gethash_ack_callback = None
        self._ble_device = None

    def replace_mqtt(self, mqtt: MammotionCloud) -> None:
        device = self._cloud_device.device
        self._cloud_device = MammotionBaseCloudDevice(mqtt, cloud_device=device, state_manager=self._state_manager)

    def has_cloud(self) -> bool:
        return self._cloud_device is not None

    def has_ble(self) -> bool:
        return self._ble_device is not None


class MammotionDeviceManager:
    devices: dict[str, MammotionMixedDeviceManager] = {}

    def add_device(self, mammotion_device: MammotionMixedDeviceManager) -> None:
        exists: MammotionMixedDeviceManager | None = self.devices.get(mammotion_device.name)
        if exists is None:
            self.devices[mammotion_device.name] = mammotion_device
            return
        if mammotion_device.has_cloud():
            exists.replace_cloud(mammotion_device.cloud())
        if mammotion_device.has_ble():
            exists.replace_ble(mammotion_device.ble())

    def get_device(self, mammotion_device_name: str) -> MammotionMixedDeviceManager:
        return self.devices.get(mammotion_device_name)

    async def remove_device(self, name: str) -> None:
        if self.devices.get(name):
            device_for_removal = self.devices.pop(name)
            loop = asyncio.get_running_loop()
            if device_for_removal.has_cloud():
                should_disconnect = {
                    device
                    for key, device in self.devices.items()
                    if device.cloud() is not None and device.cloud().mqtt == device_for_removal.cloud().mqtt
                }
                if len(should_disconnect) == 0:
                    await loop.run_in_executor(None, device_for_removal.cloud().mqtt.disconnect)
                await device_for_removal.cloud().stop()
            if device_for_removal.has_ble():
                await device_for_removal.ble().stop()

            del device_for_removal


class Mammotion:
    """Represents a Mammotion account and its devices."""

    device_manager = MammotionDeviceManager()
    mqtt_list: dict[str, MammotionCloud] = dict()

    _instance: Mammotion | None = None

    def __new__(cls, *args: Any, **kwargs: Any):
        if not cls._instance:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self) -> None:
        """Initialize MammotionDevice."""
        self._login_lock = asyncio.Lock()

    async def login_and_initiate_cloud(self, account, password, force: bool = False) -> None:
        async with self._login_lock:
            exists: MammotionCloud | None = self.mqtt_list.get(account)
            if not exists or force:
                cloud_client = await self.login(account, password)
                await self.initiate_cloud_connection(account, cloud_client)

    async def refresh_login(self, account: str, password: str | None = None) -> None:
        async with self._login_lock:
            exists: MammotionCloud | None = self.mqtt_list.get(account)
            if not exists:
                return
            mammotion_http = exists.cloud_client.mammotion_http
            await mammotion_http.refresh_login(account, password)
            await self.connect_iot(exists.cloud_client)

            if not exists.is_connected():
                loop = asyncio.get_running_loop()
                await loop.run_in_executor(None, exists.connect_async)

    async def initiate_cloud_connection(self, account: str, cloud_client: CloudIOTGateway) -> None:
        loop = asyncio.get_running_loop()
        if mqtt := self.mqtt_list.get(account):
            if mqtt.is_connected():
                await loop.run_in_executor(None, mqtt.disconnect)

        mammotion_cloud = MammotionCloud(
            MammotionMQTT(
                region_id=cloud_client.region_response.data.regionId,
                product_key=cloud_client.aep_response.data.productKey,
                device_name=cloud_client.aep_response.data.deviceName,
                device_secret=cloud_client.aep_response.data.deviceSecret,
                iot_token=cloud_client.session_by_authcode_response.data.iotToken,
                client_id=cloud_client.client_id,
                cloud_client=cloud_client,
            ),
            cloud_client,
        )
        self.mqtt_list[account] = mammotion_cloud
        self.add_cloud_devices(mammotion_cloud)

        await loop.run_in_executor(None, self.mqtt_list[account].connect_async)

    def add_cloud_devices(self, mqtt_client: MammotionCloud) -> None:
        for device in mqtt_client.cloud_client.devices_by_account_response.data.data:
            mower_device = self.device_manager.get_device(device.deviceName)
            if device.deviceName.startswith(("Luba-", "Yuka-")) and mower_device is None:
                mixed_device = MammotionMixedDeviceManager(
                    name=device.deviceName,
                    iot_id=device.iotId,
                    cloud_client=mqtt_client.cloud_client,
                    cloud_device=device,
                    mqtt=mqtt_client,
                    preference=ConnectionPreference.WIFI,
                )
                mixed_device.state.mower_state.product_key = device.productKey
                mixed_device.state.mower_state.model = (
                    device.productName if device.productModel is None else device.productModel
                )
                self.device_manager.add_device(mixed_device)
            elif device.deviceName.startswith(("Luba-", "Yuka-")) and mower_device:
                if mower_device.cloud() is None:
                    mower_device.add_cloud(mqtt=mqtt_client)
                else:
                    mower_device.replace_mqtt(mqtt_client)

    def set_disconnect_strategy(self, disconnect: bool) -> None:
        for device_name, device in self.device_manager.devices.items():
            if device.ble() is not None:
                ble_device: MammotionBaseBLEDevice = device.ble()
                ble_device.set_disconnect_strategy(disconnect)

    async def login(self, account: str, password: str) -> CloudIOTGateway:
        """Login to mammotion cloud."""
        mammotion_http = MammotionHTTP()
        cloud_client = CloudIOTGateway(mammotion_http)
        await mammotion_http.login(account, password)
        await self.connect_iot(cloud_client)
        return cloud_client

    @staticmethod
    async def connect_iot(cloud_client: CloudIOTGateway) -> None:
        mammotion_http = cloud_client.mammotion_http
        country_code = mammotion_http.login_info.userInformation.domainAbbreviation
        await cloud_client.get_region(country_code)
        await cloud_client.connect()
        await cloud_client.login_by_oauth(country_code)
        await cloud_client.aep_handle()
        await cloud_client.session_by_auth_code()
        await cloud_client.list_binding_by_account()

    async def remove_device(self, name: str) -> None:
        await self.device_manager.remove_device(name)

    def get_device_by_name(self, name: str) -> MammotionMixedDeviceManager:
        return self.device_manager.get_device(name)

    def get_or_create_device_by_name(self, device: Device, mqtt_client: MammotionCloud) -> MammotionMixedDeviceManager:
        if mow_device := self.device_manager.get_device(device.deviceName):
            return mow_device
        mow_device = MammotionMixedDeviceManager(
            name=device.deviceName,
            iot_id=device.iotId,
            cloud_client=mqtt_client.cloud_client,
            mqtt=mqtt_client,
            cloud_device=device,
            ble_device=None,
        )
        self.device_manager.add_device(mow_device)
        return mow_device

    async def send_command(self, name: str, key: str):
        """Send a command to the device."""
        device = self.get_device_by_name(name)
        if device:
            if device.preference is ConnectionPreference.BLUETOOTH and device.has_ble():
                return await device.ble().command(key)
            if device.preference is ConnectionPreference.WIFI:
                return await device.cloud().command(key)
            # TODO work with both with EITHER
        return None

    async def send_command_with_args(self, name: str, key: str, **kwargs: Any):
        """Send a command with args to the device."""
        device = self.get_device_by_name(name)
        if device:
            if device.preference is ConnectionPreference.BLUETOOTH and device.has_ble():
                return await device.ble().command(key, **kwargs)
            if device.preference is ConnectionPreference.WIFI:
                return await device.cloud().command(key, **kwargs)
            # TODO work with both with EITHER
        return None

    async def start_sync(self, name: str, retry: int):
        device = self.get_device_by_name(name)
        if device:
            if device.preference is ConnectionPreference.BLUETOOTH and device.has_ble():
                return await device.ble().start_sync(retry)
            if device.preference is ConnectionPreference.WIFI:
                return await device.cloud().start_sync(retry)
            # TODO work with both with EITHER
        return None

    async def start_map_sync(self, name: str):
        device = self.get_device_by_name(name)
        if device:
            if device.preference is ConnectionPreference.BLUETOOTH and device.has_ble():
                return await device.ble().start_map_sync()
            if device.preference is ConnectionPreference.WIFI:
                return await device.cloud().start_map_sync()
            # TODO work with both with EITHER
        return None

    async def get_stream_subscription(self, name: str, iot_id: str) -> Response[StreamSubscriptionResponse] | Any:
        device = self.get_device_by_name(name)
        if DeviceType.is_mini_or_x_series(name):
            _stream_response = await device.mammotion_http.get_stream_subscription_mini_or_x_series(
                iot_id, DeviceType.is_yuka(name) and not DeviceType.is_yuka_mini(name)
            )
            _LOGGER.debug(_stream_response)
            return _stream_response
        else:
            _stream_response = await device.mammotion_http.get_stream_subscription(iot_id)
            _LOGGER.debug(_stream_response)
            return _stream_response

    async def get_video_resource(self, name: str, iot_id: str) -> Response[VideoResourceResponse] | None:
        device = self.get_device_by_name(name)

        if DeviceType.is_mini_or_x_series(name):
            _video_resource_response = await device.mammotion_http.get_video_resource(iot_id)
            _LOGGER.debug(_video_resource_response)
            return _video_resource_response
        return None

    def mower(self, name: str) -> MowingDevice | None:
        device = self.get_device_by_name(name)
        if device:
            return device.state
        return None



class tmp_constant:
    ALGECC = "ecc"
    ALGHMACMD5 = "hmacmd5"
    ALGNONE = "none"
    ALINK_SECURE_URI_SCHEME = "coaps"
    ALINK_URI_SCHEME = "coap"
    AUTH_VER = "1.0"
    CONFIG_TYPE_CLIENT = "ClientAuthInfo"
    CONFIG_TYPE_PROVISION_RECEIVER = "SetupData"
    CONFIG_TYPE_SERVER = "ServerAuthInfo"
    DATA_KEY_DEVICENAME = "MAC"
    DATA_KEY_DEVICE_WIFI_STATUS = "wifiEnabled"
    DEFAULT_COAP_SECURE_PORT = 5684
    DEFAULT_COAP_UDP_PORT = 5683
    DEFAULT_MULTI_CAST_ADDRESS = "224.0.1.187"
    DEVICES = "devices"
    DEVICE_DATA_FORMAT_ALINK = "ALINK_FORMAT"
    DEVICE_IOTID = "iotId"
    DEVICE_MODEL = "deviceModel"
    DEVICE_MODEL_EVENTS = "events"
    DEVICE_MODEL_PROPERTIES = "properties"
    DEVICE_MODEL_SERVICES = "services"
    DEVICE_MODEL_TYPE = "model"
    DEVICE_NAME = "deviceName"
    DEVICE_NAME_POSITION = 2
    DEVICE_PRODUCT_KEY = "productKey"
    DEVICE_PROFILE = "profile"
    EVENT_PROPERTY_URI_POST = "post"
    EVENT_PROPERTY_URI_POST_REPLY = "post_reply"
    EVENT_PROPERTY_URI_PRE = "thing/event/property"
    EVENT_URI_POST = "post"
    EVENT_URI_PRE = "thing/event"
    EXPAND_SPLITE = "."
    GROUP_CLOUD_ROLE_COTROLLER = "COTROLLER"
    GROUP_CLOUD_ROLE_DEVICE = "DEVICE"
    GROUP_DEVICE_LOCAL_STATUS_OFFLINE = 3
    GROUP_DEVICE_LOCAL_STATUS_ONLINE = 1
    GROUP_LOCAL_SATUS_ALLOFFLINE = 2
    GROUP_LOCAL_SATUS_ALLONLINE = 0
    GROUP_LOCAL_SATUS_PARTONLINE = 1
    GROUP_OP_ADD = "add"
    GROUP_OP_DEL = "del"
    GROUP_ROLE_CONTROLLER = "cloud_ctl"
    GROUP_ROLE_DEVICE = "cloud_dev"
    GROUP_ROLE_UNKNOWN = "null"
    HEARTBEAT_DEFAULT_TIME = 600000
    HEARTBEAT_MAX_RETRY_TIME = 3
    IDENTIFIER_DEVICENOTIFY = "devnotify"
    IDENTIFIER_RAW_DATA_DOWN = "raw_data_down"
    IDENTIFIER_SETUP = "setup"
    IDENTITY_ADMIN = "0"
    IDENTITY_GUEST = "1"
    IDENTITY_PROVISION = "2"
    KEY_CLIENT_ID = "clientId"
    KEY_IOT_NeedRsp = "NeedRsp"
    KEY_IOT_NeedRsp_EX = "NeedRspEx"
    KEY_IOT_PERFORMANCE_EVENT_REQ = "req"
    KEY_IOT_PERFORMANCE_EVENT_RES = "res"
    KEY_IOT_PERFORMANCE_ID = "IotPerformanceId"
    KEY_IOT_QOS = "QosLevel"
    KEY_IOT_QOS_EX = "QosLevelEx"
    KEY_SIGN_METHOD = "signMethod"
    KEY_SIGN_VALUE = "sign"
    LINK_DEFAULT_CONNECT_ID = "LINK_DEAFULT"
    MAX_BLOCK_SIZE = 3072
    MAX_DEVICE_MODEL_LENGTH = 3072
    METHOD_AUTH_USER = "core.service.user"
    METHOD_DEVICE_NOTIFY = "core.service.notify"
    METHOD_HEART_BEAT = "core.service.heartBeat"
    METHOD_IDENTIFIER_DEV = "dev"
    METHOD_PROPERTY_GET = "thing.service.property.get"
    METHOD_PROPERTY_POST = "thing.event.property.post"
    METHOD_PROPERTY_SET = "thing.service.property.set"
    METHOD_SERVICE_AUTH = "core.service.auth"
    METHOD_SERVICE_AUTH_INFO = "core.service.auth_info"
    METHOD_SERVICE_DISCOVERY = "core.service.dev"
    METHOD_SERVICE_PRE = "thing.service."
    METHOD_SETUP = "core.service.setup"
    METHOD_TSL_TEMPLATE = "thing.dsltemplate.get"
    METHOD_URI_PRE = "thing/service"
    METHOD_USER_SERVICE = "core.service.user"
    MODEL_TYPE_ALI_LCA_CLOUD = "4"
    MODE_VALUE_RANDW = "rw"
    MODE_VALUE_READ = "r"
    MODE_VALUE_WRITE = "w"
    MQTT_NOTIFY_TOPIC_PREFIX = "/app/down/_thing"
    MQTT_TOPIC_EVENTS = "/app/down/thing/events"
    MQTT_TOPIC_NOTIFY = "/app/down/_thing/event/notify"
    MQTT_TOPIC_PREFIX = "/app/down/thing"
    MQTT_TOPIC_PROPERTIES = "/app/down/thing/properties"
    MQTT_TOPIC_SERVICE_REPLY = "/app/down/thing/service/invoke/reply"
    MQTT_TOPIC_STATUS = "/app/down/thing/status"
    OPTIONNUMREGISTRY_ACCESSKEY = 36  # deprecated
    PATH_DISCOVERY = "/dev/core/service/dev"
    PATH_GROUP_PRE = "/thing/service/"
    PATH_NOTIFY = "/dev/core/service/notify"
    PATH_SETUP = "/core/service/setup"
    PAYLOAD_CODE = "code"
    PRODUCT_KEY_POSITION = 1
    PROPERTY_IDENTIFIER = "identifier"
    PROPERTY_IDENTIFIER_GET = "get"
    PROPERTY_IDENTIFIER_SET = "set"
    PROPERTY_NAME = "name"
    PROPERTY_TIME = "time"
    PROPERTY_URI_PRE = "thing/service/property"
    PROPERTY_VALUE = "value"
    REQUEST_DELAYTIME = "delayTime"
    REQUEST_ID = "id"
    REQUEST_METHOD = "method"
    REQUEST_PARAMS = "params"
    REQUEST_SESSIONKEY = "sessionKey"
    REQUEST_VERSION = "version"
    RESULT_FAIL = "Failed"
    RESULT_OK = "Success"
    RRESPONSE_DATA = "data"
    RRESPONSE_MESSAGEID = "messageId"
    SERVICE_CALLTYPE = "callType"
    SERVICE_DESC = "desc"
    SERVICE_IDENTIFIER = "identifier"
    SERVICE_INPUTDATA = "inputData"
    SERVICE_METHOD = "method"
    SERVICE_NAME = "name"
    SERVICE_OUTPUTDATA = "outputData"
    SESSION_VALID_TIME = 86400  # deprecated
    TAG = "[Tmp]"
    THING_WIFI_STATUS_NOTIFY = "thing/wifi/status/notify"
    TMP_MODEL_TYPE_ALI_BREEZE = "breeze"
    TMP_MODEL_TYPE_ALI_LCA_CLOUD = "cloudlca"
    TMP_MODEL_TYPE_ALI_THIRD_PART = "third"
    TMP_MODEL_TYPE_ALI_WIFI = "wifi"
    TYPE_VALUE_ARRAY = "array"
    TYPE_VALUE_BOOLEAN = "bool"
    TYPE_VALUE_DATE = "date"
    TYPE_VALUE_DOUBLE = "double"
    TYPE_VALUE_ENUM = "enum"
    TYPE_VALUE_FLOAT = "float"
    TYPE_VALUE_INTEGER = "int"
    TYPE_VALUE_STRING = "string"
    TYPE_VALUE_STRUCT = "struct"
    TYPE_VALUE_TEXT = "text"
    UNKNOWN_CODE = 0  # deprecated
    URI_ALINK_DEVICE_DATA = "alink/deviceStatus"  # deprecated
    URI_ALINK_SERVICE = "alink/service"  # deprecated
    URI_AUTH = "/auth"  # deprecated
    URI_AUTHEN_REGISTER = "/sys/{productKey}/{deviceName}/thing/authen/sub/register"
    URI_AUTHEN_REGISTER_REPLY = "/sys/{productKey}/{deviceName}/thing/authen/sub/register_reply"
    URI_BLACKLIST_UPDATE_POST = "/thing/lan/blacklist/update"
    URI_BLACKLIST_UPDATE_REPLY_POST = "/thing/lan/blacklist/update_reply"
    URI_DEVICE = "/device/core/dev"
    URI_DEV_PATH_PRE = "dev"
    URI_METHOD = "/device/core/method"
    URI_MODEL = "/model"
    URI_PATH_SPLITER = "/"
    URI_PERMISSION = "/permission"
    URI_PREFIX_ALCS_SERVICE = "core/service"
    URI_PREFIX_UPDATE_POST = "/thing/lan/prefix/update"
    URI_PREFIX_UPDATE_REPLY_POST = "/thing/lan/prefix/update_reply"
    URI_PRODUCT_DEVICE_REPLACE = "{deviceName}"
    URI_PRODUCT_PRODUCT_REPLACE = "{productKey}"
    URI_PROPERTY_SET = "/thing/service/property/set"
    URI_SYS_PATH_PRE = "sys"
    URI_THING = "/thing"
    URI_TOPIC_LOCALDEVICE_STATECHANGE = "group/localstatechange"
    URI_TOPIC_REPLY_POST = "_reply"
    URI_UPDATE_DEVICE_INFO = "/sys/{productKey}/{deviceName}/thing/deviceinfo/update"
    URI_UPDATE_DEVICE_INFO_REPLY = "/sys/{productKey}/{deviceName}/thing/deviceinfo/update_reply"
    URI_USER = "/user"
    VALUE_SHA256 = "sha256"
    VERSION = "1.0"
    WIFI_STATUS_KEY = "status"
    WIFI_STATUS_NOTIFY_PARAMS = "params"
